
///////////////////////////////
// Extend the leaflet layers //
///////////////////////////////
var baseFeature =
{
  _visible: true,
  _initEvents: function () {
    //this is an leaflet override to allow hideing/showing features
    if (this._visible == true) this.show();
    else this.hide();

    if (this.options.clickable) {
      if (L.Browser.svg || !L.Browser.vml) {
        L.DomUtil.addClass(this._path, 'leaflet-clickable');
      }

      L.DomEvent.on(this._container, 'click', this._onMouseClick, this);

      var events = ['dblclick', 'mousedown', 'mouseover', 'mouseout', 'mousemove', 'contextmenu'];
      for (var i = 0; i < events.length; i++) {
        L.DomEvent.on(this._container, events[i], this._fireMouseEvent, this);
      }

    }
  },
  hide: function () {
    if (this._path) L.DomUtil.addClass(this._path, 'leaflet-hide');
    this._visible = false;
  },
  show: function () {
    if (this._path) L.DomUtil.removeClass(this._path, 'leaflet-hide');
    this._visible = true;
  }
};

var PointFeature = L.CircleMarker.extend(baseFeature).extend({
  _default_style: null,
  SetDefaultStyle: function (default_style) {
    this._default_style = default_style;
  },
  SetToDefaultStyle: function () {
    this.setStyle(this._default_style); //first reset style
  },
  GetColor: function () {
    return this.options.fillColor;
  },
  Select: function () {
    this.setStyle(this._default_style); //first reset style
    this.setRadius(this.getRadius() + 5 * Math.sqrt(screenDensity));
    this.setStyle({
      weight: this.options.weight + (1 * Math.sqrt(screenDensity)),
      color: '#FFF'
    });
    this.bringToFront();
  },
  Deselect: function () {
    this.setRadius(this.getRadius() - 5 * Math.sqrt(screenDensity));
    this.setStyle({
      weight: this.options.weight - (1 * Math.sqrt(screenDensity)),
      color: this.color
    });
  }
});

var PolygonFeature = L.Polygon.extend(baseFeature).extend({
  _default_style: null,
  SetDefaultStyle: function (default_style) {
    this._default_style = default_style;
  },
  SetToDefaultStyle: function () {
    this.setStyle(this._default_style); //first reset style
  },
  GetColor: function () {
    return this.options.fillColor;
  },
  Select: function () {
    this.setStyle(this._default_style); //first reset style
    this.setStyle({
      weight: this.options.weight + (2 * Math.sqrt(screenDensity)),
      fillOpacity: 0.5
    });
    this.bringToFront();
  },
  Deselect: function () {
    this.setStyle(this._default_style);
    this.bringToBack();
  }
});

var LineFeature = L.Polyline.extend(baseFeature).extend({
  _default_style: null,
  SetDefaultStyle: function (default_style) {
    this._default_style = default_style;
  },
  SetToDefaultStyle: function () {
    this.setStyle(this._default_style);
  },
  GetColor: function () {
    return this.options.color;
  },
  Select: function () {
    this.setStyle(this._default_style);
    this.setStyle({
      weight: this.options.weight + (2 * Math.sqrt(screenDensity)),
      fillOpacity: 0.5
    });
    this.bringToFront();
  },
  Deselect: function () {
    this.setStyle(this._default_style);
  }
});

