L.TileLayer.GeoPackage = L.TileLayer.extend({
  options: {
    minZoom: 0,
    maxZoom: 19,
    tileSize: 256,
    subdomains: 'abc',
    errorTileUrl: '',
    attribution: 'Map data © OpenStreetMap contributors',
    opacity: 1,
    scheme: 'xyz',
    noWrap: false,
    reuseTiles: false,
    unloadInvisibleTiles: L.Browser.mobileWebkit,
    updateWhenIdle: L.Browser.mobileWebkit
  },

  initialize: function (url, tilestorage, options) {
    this._url = "";
    if (typeof this.options.subdomains == 'string') {
      this.options.subdomains = this.options.subdomains.split('');
    }
    L.Util.setOptions(this, options);
  },

  _loadTile: function (tile, tilePoint, zoom) {
    tile._layer = this;
    tile.onload = this._tileOnLoad;
    tile.onerror = this._tileOnError;

    this._adjustTilePoint(tilePoint);
    this.getTileUrl(tile, tilePoint, zoom);

    this.fire('tileloadstart', {
      tile: tile,
      url: tile.src
    });

  },

  getTileUrl: function (tile, tilePoint) {
    var subdomains = this.options.subdomains,
                 s = this.options.subdomains[(tilePoint.x + tilePoint.y) % subdomains.length];
    var fallback = this._url
                       .replace('{s}', s)
                       .replace('{z}', this._map.getZoom())
                       .replace('{x}', tilePoint.x)
                       .replace('{y}', tilePoint.y);

    SyncCallback('callback://requestTile/' + this._map.getZoom() + "/" + tilePoint.x + "/" + tilePoint.y,
      function (a) {
        if (a)
          tile.src = 'data:image/png;base64,' + a;
        else tile.src = L.Util.emptyImageUrl;

      });
  }
});
