"use strict";
var map = null;

var locationCursor;
var locationCursorAddedToMap = false;
var infoWindow;
var infoWindowShown = false;

var directionLocationCursorAngle = 0;
var directionLocationCursor = null;

var selectedFeature = null; //currently selected feature

var screenDensity = 1.0;

var layerGroups = {}; //based on layer name!! should be id??
var features = {}; //Create a map of all features (layers) to look up with featureId
var provider = null;

////////////////////
// Offline Layers //
////////////////////
var _offlinemap = new L.TileLayer.GeoPackage();
function SetOfflineMapOptions(options)
{
  // method for conveniently merging options
  L.Util.setOptions(_offlinemap, options);
}

//////////////////////////////
// General public functions //
//////////////////////////////

//***This file has been setup to test Leaflet
//It was modified form an existing GoogleMapApi.js functions file
//Several of the pre-existing Google functions have been left as comments for reference.

var availible_providers = {
  "Google": function(m) { return new GoogleProvider(m); },
  "Esri": function (m) { return new EsriProvider(m); },
  "OpenStreetMap": function (m) { return new OpenStreetMapProvider(m); },
  "Offline": function (m) {

    if ((provider != null))
      if (m.getZoom() > 16 && provider.name != "Offline")
    {
      m.setZoom(16); //only reset the zoom if above 16 and switched from another provider
    }
    return new OfflineProvider(m);
  }
}

function UseMapProvider(providername, layername) {
  if (!provider) {
    provider = availible_providers[providername](map);
  }
  else if (provider.name != providername)
  {
    provider.Shutdown();
    provider = availible_providers[providername](map);
  }
  if(layername) provider.SetLayer(layername);
}

function SetTileProviderLayer(name) {
  provider.SetLayer(name);
}

//Return comma separated string of layers
function GetTileProviderLayerNames() {
  return Object.keys(provider.baseLayers).join();
}

function GetProvidersCurrentLayerName() {
  return provider.currentlayername;
}

function projectPoint(x, y) {
  var point = map.latLngToLayerPoint(new L.LatLng(y, x));
  this.stream.point(point.x, point.y);
}

function AddLocationMarker() {

}

function onViewreset() {
}

function setMarginTopControls(margin) {
  $(".leaflet-control-container > .leaflet-top").css("margin-top", margin + "px");
}

function setMarginBottomControls(margin) {
  $(".leaflet-control-container > .leaflet-bottom").css("margin-bottom", margin + "px");
}

function onMapClick(e) {
  //deselectFeature();
  hideInfoWindow();
  //This is a good place to put functions for testing.
  //It is possible to load the HTML file in Chrome, and use the browser for debugging.
  SyncCallback('callback://mapClick/' + coordStr(e.latlng.lat) + '/' + coordStr(e.latlng.lng));
}

function SetGnssStatusInfo(distance, units) {
  map.gpsControl.setStatusInfo(distance, units);
}

function SetGnssStatusAlert(alert) {
  map.gpsControl.setAlert(alert);
}

function SetGnssAccuracyAlert(alert) {
  map.gpsControl.setAccuracyAlert(alert);
}

function SetIconPanelVisibility(visible) {
  map.gpsControl.setIconPanelVisibility(visible);
}

function SetDeltaIconVisibility(visible) {
  map.gpsControl.setDeltaIconVisibility(visible);
}

function SetSatelliteIconVisibility(visible) {
  map.gpsControl.setSatelliteIconVisibility(visible);
}

function SetTowerIconVisibility(visible) {
  map.gpsControl.setTowerIconVisibility(visible);
}

function ToggleSatelliteIconDelta() {
  map.gpsControl.toggleSatelliteIconDelta();
}

function mapReady() {
  initializeProviders();
  
  //Called when HTML setup and ready for map initialization
  infoWindow = L.popup();
  map = L.map('map', {zoomControl: false});

  //override the behaviour of invalidateSize to stop it centring the map
  map.invalidateSize = function (options) {
    if (!this._loaded) { return this; }

    var oldSize = this.getSize();
    this._sizeChanged = true;
    this._initialCenter = null;
    var newSize = this.getSize();

    this.fire('move');
    this.fire('moveend');
    return this.fire('resize', {
      oldSize: oldSize,
      newSize: newSize
    });
  };

  SetInvisibleControl(); // Sets padding so that the google logo is displayed.
  new L.Control.Scale({ position: 'bottomleft' }).addTo(map);
  new L.Control.Zoom({ position: 'bottomleft' }).addTo(map);

  //provide a map for testing in Chrome
  if (DEBUGCHROME) { 
    UseMapProvider('Esri', 'StreetMap');
    UseMapProvider('Google', 'StreetMap');
    UseMapProvider('Google', 'Hybrid');
    map.locate({ setView: true, maxZoom: 16 });
    chromeCreateControls();
  }

  map.on('click', onMapClick);

  map.on('viewreset', onViewreset);

  map.on('baselayerchange', function (e) {
    //Make sure the zoom is not outside the limits of the max and min zoom for baselayer
    if (map.getZoom() < map.getMinZoom()) map.setZoom(map.getMinZoom());
    if (map.getZoom() > map.getMaxZoom()) map.setZoom(map.getMaxZoom());
  });


  if (!DEBUGCHROME) {
    map.on('movestart', function (event) {
      SyncCallback('callback://mapMoveStart/');
    });

    map.on('moveend', function (event) {
      var centerLatLng = map.getCenter();
      var zoomLevel = map.getZoom();
      var northEast = map.getBounds().getNorthEast();
      var southWest = map.getBounds().getSouthWest();
      var minX = Math.min(northEast.lng, southWest.lng);
      var minY = Math.min(northEast.lat, southWest.lat);
      var maxX = Math.max(northEast.lng, southWest.lng);
      var maxY = Math.max(northEast.lat, southWest.lat);

      var callback = 'callback://mapMoveEnd/' + coordStr(centerLatLng.lat) + '/' +
                                                   coordStr(centerLatLng.lng) + '/' +
                                                   zoomLevel + '/' +
                                                   minX + '/' + minY + '/' + maxX + '/' + maxY;
      SyncCallback(callback);

    });

    SyncCallback('callback://mapReady');
  } else {
    map.on('click', function (e) {
      map.addLayer(new R.Marker(e.latlng));

    });
  }
}
function setScreenDensity(density) {
  screenDensity = density;
}

// exclusivelyHideFeatures(feature_ids)
// Will hide all feature and show others
// Requires : a list of feature ids ie. [12,32,32]
// can take and empty list ie. []
function exclusivelyHideFeatures(feature_ids)
{
  for (var id in features) {
    features[id].show();
  }
  for (var i = 0; i < feature_ids.length; i++) {
    if (features[feature_ids[i]] != undefined)
      features[feature_ids[i]].hide();
  }
}

// Creates the location (GPS) cursor
function createLocationCursor() {
  var icon = L.icon({
    iconUrl: "Images/ic_location_cursor.png",
    iconSize: [18, 18],
    iconAnchor: [9, 9]
  });
  locationCursor = L.marker();
  locationCursor.setIcon(icon);
}

// sets the zoom level of the map (0 = world extent)
function setZoom(level) {
  map.setZoom(level);
}

function setFeatureBearing(bearing)
{
  /* Show on map feature direction
  if (directionLocationCursor) {
    if (bearing && selectedFeature) {
      directionLocationCursor.featureDirection(bearing, selectedFeature.GetColor(), true);
    }
    else {
      directionLocationCursor._featurePointer.hide(1000);
    }
  }*/
}

// Positions the location (GPS) cursor
function showLocation(latitude, longitude, direction) {
  var latlng = new L.LatLng(latitude, longitude);
  if (directionLocationCursor) {
    directionLocationCursor.userDirectionPosition(direction, latlng, true);
  } else {
    directionLocationCursor = new R.PositionDirectionCursor(latlng)
    map.addLayer(directionLocationCursor);
  }
}

function getCenterStr() {
  var latlng = map.getCenter();
  return coordStr(latlng.lat) + "," + coordStr(latlng.lng);
}

function setCenter(latitude, longitude) {
  var latlng = new L.LatLng(latitude, longitude);
  map.setView(latlng);
}

function setView(latitude, longitude, zoom, animate) {
  var latlng = new L.LatLng(latitude, longitude);
  map.setView(latlng, zoom, {animate: animate});
}

function panBy(x, y, animate) {
  if (animate) map.panBy(new L.Point(x, y));
  else {
    map._rawPanBy(new L.Point(x, y));
    map.fire('move').fire('moveend');
  }
}

function panTo(latitude, longitude) {
  var latlng = new L.LatLng(latitude, longitude);
  map.panTo(latlng);
}

function panToScreenLocation(screenLocationX, screenLocationY) {
  var latLong = fromScreenLocationToLatLng(screenLocationX, screenLocationY);
  _map.PanTo(latLong);
}

function fitBounds(minLat, minLng, maxLat, maxLng, animate) {
  var southWest = new L.LatLng(minLat, minLng);
  var northEast = new L.LatLng(maxLat, maxLng);

  var bounds = new L.latLngBounds(southWest, northEast);
  map.fitBounds(bounds, {animate: animate});
} 

function createImageMarker(latitude, longitude, resource, anchorX, anchorY, scaledSizeX, scaledSizeY, draggable, zOrder) {
  var markerLocation = new L.LatLng(latitude, longitude);

  var ImageIcon = L.Icon.extend({
    options: {
      iconUrl: resource,
      iconAnchor: [anchorX, anchorY],
      iconSize: [scaledSizeX, scaledSizeY]
    }
  });

  var myIcon = new ImageIcon();

  var marker = new L.Marker(markerLocation, { icon: myIcon, draggable: draggable }).addTo(map);

  marker.on('dragend', function (event) {
    SyncCallback('callback://markerDragEnd/' + coordStr(event.target._latlng.lat) + '/' + coordStr(event.target._latlng.lng));
  });

  return marker;
}

function fromLatLngToWorldPointStr(latitude, longitude) {
  var point = fromLatLngToWorldPoint(latitude, longitude);
  return coordStr(point.x) + "," + coordStr(point.y);
}

function fromWorldPointToLatLngStr(pointX, pointY) {
  var latlng = fromWorldPointToLatLng(pointX, pointY);
  return coordStr(latlng.lat) + "," + coordStr(latlng.lng);
}
 
function fromLatLngToScreenLocationStr(latitude, longitude) {
  var screenLocation = fromLatLngToScreenLocation(latitude, longitude);
  return coordStr(screenLocation.x) + "," + coordStr(screenLocation.y);
}
 
function fromScreenLocationToLatLngStr(screenLocationX, screenLocationY) {
  var latlng = fromScreenLocationToLatLng(screenLocationX, screenLocationY);
  return coordStr(latlng.lat) + "," + coordStr(latlng.lng);
}

///////////////////////////////////////////////
// Public functions to add and remove layers //
///////////////////////////////////////////////
function addPointLayer(name, zOrder, color) {
  layerGroups[name] = new PointLayer(name, zOrder, color);
  layerGroups[name].addTo(map);
}

function addPolylineLayer(name, zOrder, strokeColor, strokeWeight) {
  layerGroups[name] = new LineLayer(name, zOrder, strokeColor, strokeWeight);
  layerGroups[name].addTo(map);
}

function addPolygonLayer(name, zOrder, strokeColor, strokeWeight, fillColor) {
  layerGroups[name] = new AreaLayer(name, zOrder, strokeColor, strokeWeight, fillColor);
  layerGroups[name].addTo(map);
}

function clearMap() {
  for (var layerName in layerGroups) {
    map.removeLayer(layerGroups[layerName]);
  }
  layerGroups = {};
  features = {};
  deselectFeature(); //seems a little pointless ??
}

function redrawMap()
{
  map._resetView(map.getCenter(), map.getZoom());
}

/////////////////////////////////////////////////
// Public functions to add and remove features //
/////////////////////////////////////////////////

function addPointImageMarker(layerName, latitude, longitude, resource, anchorX, anchorY, scaledSizeX, scaledSizeY, draggable) {
  var marker = createImageMarker(latitude, longitude,
                                 resource,
                                 anchorX, anchorY,
                                 scaledSizeX, scaledSizeY,
                                 draggable,
                                 0);
  layerGroups[layerName].addLayer(marker);
}

function addPointFeature(layerName, latitude, longitude, featureId, clickable, visible) {
  var template = layerGroups[layerName].template;
  var point = createCircleMarker(latitude, longitude,
                                  template.color,
                                  featureId,
                                  template.zOrder,
                                  clickable
                                  );

  if (visible == false) point.hide();

  layerGroups[layerName].addLayer(point);
  point.color = template.color;
  features[featureId] = point;
  point.SetDefaultStyle(template);
}

function addPolylineFeature(layerName, path, featureId, clickable, visible) {
  var template = layerGroups[layerName].template;
  var polyline = createPolyline(path,
                                template.color,
                                template.weight,
                                featureId,
                                template.zOrder,
                                clickable);

  if (visible == false) polyline.hide();

  layerGroups[layerName].addLayer(polyline);
  features[featureId] = polyline;
  polyline.SetDefaultStyle(template);
}

function addPolygonFeature(layerName, path, featureId, clickable, visible) {
  var template = layerGroups[layerName].template;
  var polygon = createPolygon(path,
                              template.opacity,
                              template.color,
                              template.weight,
                              template.fillColor,
                              template.fillOpacity,
                              featureId,
                              template.zOrder,
                              clickable);

  if (visible == false) polygon.hide();

  layerGroups[layerName].addLayer(polygon);
  features[featureId] = polygon;
  polygon.SetDefaultStyle(template);
}

function removeFeatures(layerName) {
  layerGroups[layerName].clearLayers();
}

function removeFeature(featureId) {
  map.removeLayer(features[featureId]);
}

////////////////////////////////////////////////
// Feature selection and InfoWindow functions //
////////////////////////////////////////////////
function UpdateFeature(featureId, isPoint, path) {
  if (!path) {
    removeFeature(featureId);
  } else {
    if (isPoint == 1) {
      features[featureId].setLatLng(convertToLatLngArray(path)[0]);
    } else {
      features[featureId].setLatLngs(convertToLatLngArray(path));
    }
  }
}

function UpdateExistingFeature(featureId, visible, geometry)
{
  var feature = features[featureId];
  //debug(featureId);
  if (geometry == null) {
    feature.hide(); // there is no geometery
   // map.removeLayer(feature);
  } else {

    if (visible) feature.show();
      else feature.hide();

    if (feature instanceof PointFeature) {
      features[featureId].setLatLng(convertToLatLngArray(geometry)[0]); //a point feature requires a point
    }
    else {
      features[featureId].setLatLngs(convertToLatLngArray(geometry));
    }
  }
}

function ReorderLayers() {
  RezOrderLayers(layerGroups); //puts area, line and points in order
}

//handle the options here this is extendable you can add anything you want
//pass options in a form string ie. visibilty:true, color:blue etc.
//and extend here
//can only happen if map is visible
function FeatureOptions(featureId, options) {

 // console.log("options: " + options);
  eval("options={" + options + "}");

  if (options["visibility"]) {
    if (options["visibility"] == true) features[featureId].show()
      else features[featureId].hide();
  }
}

function FeatureIdRevertToDefaultOptions(featureId) {
  var feature = features[featureId];
  if (feature.default_options)
    feature.setStyle(feature.default_options);
}

///////////////////////
// Feature selection //
///////////////////////
function selectFeature(featureId) {
  if (selectedFeature != null) {
    deselectFeature();
  }

  var feature = features[featureId];
  if (feature) {
    feature.Select();
    selectedFeature = feature;
  }
}

function deselectFeature() {
  if (selectedFeature != null) {
    selectedFeature.Deselect(); //deselect old
    selectedFeature = null;
    ReorderLayers();
    
  }
  hideInfoWindow();
}

function removeFeature(featureId) {
  var feature = features[featureId];
  if (feature) {
    map.removeLayer(feature);
    features[featureId] = null;
    delete features[featureId];
  }
}

function hideFeature(featureId) {
  var feature = features[featureId];
  if (feature) {
    feature.hide();
  }
}

function showFeature(featureId) {
  var feature = features[featureId];
  if (feature) {
    feature.show();
  }
}

function visibilityFeature(featureId, visible) {
  var feature = features[featureId];
  if (feature) {
    if (visible) feature.show();
      else feature.hide();
  }
}

function showFeature(featureId) {
  var feature = features[featureId];
  if (feature) {
    feature.hide();
  }
}

//////////////////////////
// InfoWindow selection //
//////////////////////////
function showInfoWindow(htmlContent, latitude, longitude, featureId) {
    if (infoWindowShown) {
        map.closePopup(infoWindow);
        infoWindowShown = false;
    }

    htmlContent = "<div onclick=\"onInfoWindowClick('" + featureId + "');\">" + htmlContent + "</div>";
    infoWindow.setContent(htmlContent);
    infoWindow.setLatLng(new L.LatLng(latitude, longitude));
    infoWindow.options.offset = new L.Point(0, -4);

    map.openPopup(infoWindow);

    infoWindowShown = true;
}

function hideInfoWindow() {
  if (infoWindowShown) {
    map.closePopup(infoWindow);
    infoWindowShown = false;
  }
}

function onInfoWindowClick(featureId) {
    hideInfoWindow();
    SyncCallback('callback://infoWindowClick/' + featureId);
}

//////////////////////////////
// feature helper functions //
//////////////////////////////
function createPinMarker(latitude, longitude, color, featureId, zOrder, clickable) {
  //var image = new google.maps.MarkerImage("http://chart.googleapis.com/chart?chst=d_map_pin_letter&chld=|" + color + "|", new google.maps.Size(21, 34), null, null, new google.maps.Size(21 * pointScaleFactor, 34 * pointScaleFactor));
  //var image = { url: "http://chart.googleapis.com/chart?chst=d_map_pin_letter&chld=|" + color + "|", size: new google.maps.Size(21, 34), origin: null, anchor: null, scaledSize: new google.maps.Size(21 * pointScaleFactor, 34 * pointScaleFactor) }; // KEA-9649

  // drop the anchor now, after we've scaled
  //setMarkerImageAnchor(image);

  var markerLocation = new L.LatLng(latitude, longitude);

  var urlString = 'http://chart.googleapis.com/chart?chst=d_map_pin_letter&chld=|' + color + '|';

  var MarkerIcon = L.Icon.extend({
    options: {
      iconUrl: urlString,
      iconSize: [21, 34],
      iconAnchor: [10.5, 34]
    }
  });

  var myIcon = new MarkerIcon();

  //setMarkerImageAnchor(myIcon);

  var marker = new L.Marker(markerLocation, { icon: myIcon, zIndexOffset: zOrder, clickable: clickable }).addTo(map);
  marker.featureId = featureId;

  if (clickable) {
    marker.on('click', function (event) {
      var callback = 'callback://featureClick/' + this.featureId + '/' +
                                             coordStr(event.latlng.lat) + '/' +
                                             coordStr(event.latlng.lng);
      SyncCallback(callback)
    });
  }

  return marker;
}

function createPointMarker(latitude, longitude, color, featureId, zOrder, clickable) {
  var bounds = [[latitude, longitude], [latitude+0.0005, longitude+0.00001]];

  var marker = L.rectangle(bounds, {
    color: color,
    weight: 1,
    opacity: 1,
    fillOpacity: 1,
  });

  if (clickable) {
    marker.on('click', function (event) {
      var callback = 'callback://featureClick/' + this.featureId + '/' +
                                             coordStr(event.latlng.lat) + '/' +
                                             coordStr(event.latlng.lng);
      SyncCallback(callback);
    });
  }
  
  marker.color = color;
  return marker;
}

function createCircleMarker(latitude, longitude, color, featureId, zOrder, clickable) {
  var markerLocation = new L.LatLng(latitude, longitude);
  var feature = new PointFeature(markerLocation, {
    strokeWeight: 1,
    color: color,
    fillColor: color,
    radius: 4 * Math.sqrt(screenDensity),
    opacity: 1.0,
    fillOpacity: 1.0
  });

  feature.featureId = featureId;
  /*
  if (clickable) {
   
    feature.on('click', function (event) {
      selectPoint(this);
      
      var callback = 'callback://featureClick/' + this.featureId + '/' +
                                             coordStr(event.latlng.lat) + '/' +
                                             coordStr(event.latlng.lng);
      SyncCallback(callback);
      
    });
  }*/
  return feature;
}

function createPolyline(path, strokeColor, strokeWeight, featureId, zOrder, clickable) {
  var polyPath = convertToLatLngArray(path);

  var feature = new LineFeature(polyPath, {
    stroke: true,
    color: strokeColor,
    weight: strokeWeight * Math.sqrt(screenDensity),
    opacity: 1.0,
    clickable: clickable
  });

 // feature.default_options = 
  //Need to add this extra feature
  //zIndex: zOrder,
  feature.featureId = featureId;
  /*
  if (clickable) {
    polyline.on('click', function (event) {
      selectPoly(this);
      var callback = 'callback://featureClick/' + this.featureId + '/' +
                                           coordStr(event.latlng.lat) + '/' +
                                           coordStr(event.latlng.lng);
      SyncCallback(callback);
    });
  }*/

  return feature;
}

function createPolygon(path, strokeOpacity, strokeColor, strokeWeight, fillColor, fillOpacity, featureId, zOrder, clickable) {
  var polyPath = convertToLatLngArray(path);

  var feature = new PolygonFeature(polyPath, {
    stroke: true,
    color: strokeColor,
    weight: strokeWeight,
    fillColor: fillColor,
    fillOpacity: fillOpacity,
    clickable: clickable,
    opacity: strokeOpacity
  });

  feature.featureId = featureId;
  /*
  if (clickable) {

    polygon.on('click', function (event) {
      selectPoly(this);
 
      var callback = 'callback://featureClick/' + this.featureId + '/' +
                                           coordStr(event.latlng.lat) + '/' +
                                           coordStr(event.latlng.lng);
                                           
      SyncCallback(callback);
    });
  }*/
  return feature;
}

///////////////////////////////
// Internal helper functions //
///////////////////////////////
function fromLatLngToWorldPoint(latitude, longitude) {
  var latlng = new L.LatLng(latitude, longitude);
  return map.options.crs.projection.project(latlng);
}

function fromWorldPointToLatLng(pointX, pointY) {
  var point = new L.Point(pointX, pointY);
  return map.options.crs.projection.unproject(point);
}
 
function fromLatLngToScreenLocation(latitude, longitude) {
  // Calculate the bounds of the current visible extents in projected world coordinates.
  var northEast = map.options.crs.projection.project(map.getBounds().getNorthEast());
  var southWest = map.options.crs.projection.project(map.getBounds().getSouthWest());
  // The map can be rotated, so northEast is not necessarily top-right!
  var minX = Math.min(northEast.x, southWest.x);
  var minY = Math.min(northEast.y, southWest.y);
  var scale = Math.pow(2, map.getZoom());

  // Now convert the lat/long to a world point, then to a screen location.
  var latlng = new L.LatLng(latitude, longitude);
  var worldPoint = fromLatLngToWorldPoint(latitude, longitude);
  var screenLocation = new L.Point((worldPoint.x - minX) * scale, (worldPoint.y - minY) * scale);
  return screenLocation;
}
 
function fromScreenLocationToLatLng(screenLocationX, screenLocationY) {
  // Calculate the bounds of the current visible extents in projected world coordinates.
  var northEast = map.options.crs.projection.project(map.getBounds().getNorthEast());
  var southWest = map.options.crs.projection.project(map.getBounds().getSouthWest());
  // The map can be rotated, so northEast is not necessarily top-right!
  var minX = Math.min(northEast.x, southWest.x);
  var minY = Math.min(northEast.y, southWest.y);
  var scale = Math.pow(2, map.getZoom());

  // Now convert the screen location to a world point, then to a lat/long.
  var worldPoint = new L.Point(screenLocationX / scale + minX, screenLocationY / scale + minY);
  var latlng = map.options.crs.projection.unproject(worldPoint);
  return latlng;
}

//TODO: what is leaflet equivalent of MCVArray??
// converts path (a json string) into an MVCArray of google LatLngs
function convertToLatLngArray(path) {
  var jsonObject = eval('(' + path + ')');
  var polyPath = [];
  for (var i = 0; i < jsonObject.coords.length; i++) {
    polyPath.push(new L.LatLng(jsonObject.coords[i].lat, jsonObject.coords[i].lng));
  }
  return polyPath;  
}

// sets image's anchor to be the bottom, middle of image's scaled size
function setMarkerImageAnchor(icon) {
  var x = icon.width / 2;
  var y = icon.height;
  icon.iconAnchor = new L.Point(x, y);
}

// returns coord as a string with 7 decimal places
function coordStr(coord) {
  return coord.toFixed(7);
}

function debug(message) {
  SyncCallback('callback://debug/' + 'JAVASCRIPT DEBUG --' + message);
}

function debug_object(obj) {
  SyncCallback('callback://debug/' + 'JAVASCRIPT DEBUG --' + JSON.stringify(obj));
}

function SetupMyLocationButton() {
  var myLocationButton = L.control({ position: 'topright' });
  myLocationButton.onAdd = function () {//When added to the map below the html element is created
      this._div = L.DomUtil.create('div', 'mylocation');//mylocation is defined in the html - uses its style.
      var myLocationButtonImage = L.DomUtil.create('img', 'none', this._div);
      myLocationButtonImage.setAttribute('src', 'Images/mylocation_normal.png'); //Should put full filename in Leaflet.cs?
      myLocationButtonImage.setAttribute('width', '100%');
      myLocationButtonImage.setAttribute('height', '100%');
      L.DomEvent.addListener(this._div, 'click', L.DomEvent.stop);
      L.DomEvent.addListener(this._div, 'mousedown', function () { L.DomUtil.addClass(myLocationButton._div, 'mylocation-on'); });
      L.DomEvent.addListener(this._div, 'mouseup', function () { L.DomUtil.removeClass(myLocationButton._div, 'mylocation-on'); });
      L.DomEvent.addListener(this._div, 'click', function () {
      SyncCallback('callback://mylocation');
    });
    L.DomEvent.disableClickPropagation(this._div);
    return this._div;
  };
  map.addControl(myLocationButton);
}

function SetInvisibleControl() {
  var invisibleControl = L.control({ position: 'bottomleft' });
  invisibleControl.onAdd = function () {//When added to the map below the html element is created
    this._div = L.DomUtil.create('div', 'invisible-control');//mylocation is defined in the html - uses its style.
    return this._div;
  };
  map.addControl(invisibleControl);
}

function ToggleDirectionalCursor()
{
  directionLocationCursor.toggle_visibility();
}

// Creates an invisible marker to stop iOS crashing FLEX-746
function drawInvisibleMarker() {
  var marker = new L.CircleMarker(new L.LatLng(0, 0), {
    strokeWeight: 1,
    color: '#03f',
    fillColor: '#03f',
    radius: 1,
    opacity:0,
    fillOpacity:0
  });
  marker.addTo(map);
}

/////////////////////////


//////////////////////////
// Chrome testing       //
//////////////////////////

//Load dummy controls to do map things
//Pass the name of the control and function to be called on click two examples below
function chromeCreateControls() {
  chromeAddControl("features", chromeCreateFeatures);//Random 1000 features button
  chromeAddControl("location", chromePanTo);
}

function chromePanTo() {
  var lat = -43.54477;//Pans to trimble chch
  var lon = 172.59187;
  panTo(lat, lon);
}

//Randomly creates features around chch
function chromeCreateFeatures() {
  clearMap();
  chromeAddLayers();
  //-43.54477, 172.59187 Trimble lat/lng
  var tag = "";
  var marker;
  for (var i = 0; i < 1000; i++) {
    var lat = -43.54477 + Math.random()/10;
    var lon = 172.59187 + Math.random()/10;
    var random = Math.random();

    if (random < 0.33) {
      tag = "point" + i;
      marker = chromeCreateCircleMarker(lat, lon, '#'+Math.floor(Math.random()*16777215).toString(16), tag,2, true);
      marker.SetDefaultStyle(layerGroups["point"].template);

      layerGroups["point"].addLayer(marker);

    } else if (random <0.66) {
      tag = "line" + i;
      var path = [[lat, lon], [lat + 0.001, lon + 0.001], [lat + 0.002, lon + 0.002]];
      marker = chromeCreatePolyline(path, "#7CFC00", 3, tag, 1, true);
      marker.SetDefaultStyle(layerGroups["line"].template);
      layerGroups["line"].addLayer(marker);
    } else {
      tag = "poly" + i;
      var path = [[lat, lon], [lat + 0.001, lon + 0.001], [lat - 0.001, lon - 0.012]];
      marker = chromeCreatePolygon(path, "#FF0000", 3, "#ff6666", 0.2, tag, 0, true);
      marker.SetDefaultStyle(layerGroups["polygon"].template);
      layerGroups["polygon"].addLayer(marker);

    }
    features[tag] = marker;
  }
}

function chromeAddLayers(){
  addPointLayer("point",2,"#0000FF");//2 z-order not used
  addPolylineLayer("line",1,"#7CFC00",3);//3 line thickness
  addPolygonLayer("polygon",0,"#FF0000",3,"#ff6666",0.2);//0.2 opacity
}

function chromeAddControl(letter, callback) {
  var newControl = L.control({ position: 'topright' });
  newControl.onAdd = function () {//When added to the map below the html element is created
    this._div = L.DomUtil.create('div', 'chrome-test-control');//mylocation is defined in the html - uses its style.

    var span = L.DomUtil.create('span', 'leaflet-buttons-control-text', this._div);
    var text = document.createTextNode(letter);  //is there an L.DomUtil for this?
    span.appendChild(text);

    L.DomEvent.addListener(this._div, 'click', L.DomEvent.stop);
    L.DomEvent.addListener(this._div, 'click', callback);
    L.DomEvent.disableClickPropagation(this._div);
    return this._div;
  };

  map.addControl(newControl);
}

function chromeCreateCircleMarker(latitude, longitude, color, featureId, zOrder, clickable) {
  var markerLocation = new L.LatLng(latitude, longitude);
  var marker = new PointFeature(markerLocation, {
    strokeWeight: 1,
    color: color,
    fillColor: color,
    radius: 4 * Math.sqrt(screenDensity),
    opacity: 1.0,
    fillOpacity: 1.0
  });

  marker.featureId = featureId;

  if (clickable) {
    marker.on('click', function (event) {
      selectFeature(featureId);
      showInfoWindow("I'm a point", latitude, longitude, featureId);
    });
  }
  return marker;
}

function chromeCreatePolyline(path, strokeColor, strokeWeight, featureId, zOrder, clickable) {

  var polyline = new LineFeature(path, {
    stroke: true,
    color: strokeColor,
    weight: strokeWeight * Math.sqrt(screenDensity),
    opacity: 1.0,
    clickable: clickable
  });

  //Need to add this extra feature
  //zIndex: zOrder,


  polyline.featureId = featureId;
  

  if (clickable) {
    polyline.on('click', function (event) {
      selectFeature(featureId);
      showInfoWindow("I'm a LIIIIIIIIINE", path[0][0], path[0][1], featureId);
    });
  }

  return polyline;
}

function chromeCreatePolygon(path, strokeColor, strokeWeight, fillColor, fillOpacity, featureId, zOrder, clickable) {

  var polygon = new PolygonFeature(path, {
    stroke: true,
    color: strokeColor,
    weight: strokeWeight,
    fillColor: fillColor,
    fillOpacity: fillOpacity,
    clickable: clickable
  });

  polygon.featureId = featureId;

  if (clickable) {
    polygon.on('click', function (event) {
      selectFeature(featureId);
      showInfoWindow("I'm a polygon", path[0][0], path[0][1], featureId);
    });
  }
  return polygon;
}

//////////////////////////
// Chrome testing  end  //
//////////////////////////