function initializeProviders() {
  function Provider(map) {
    this.map = map;
    this.baseLayers = null;
    this.control = null;
    this.currentlayername = null;
    this.currentlayer = null;
    this.name = null;
  }

  Provider.prototype.Shutdown = function () {

    if(this.control) this.map.removeControl(this.control);
    for (var i in this.baseLayers)
      this.map.removeLayer(this.baseLayers[i]);
    this.currentlayername = null;
  }

  Provider.prototype.SetLayer = function (name) {
    var layer = this.baseLayers[name];
    if (layer && (this.currentlayername != name)) {
      if (this.currentlayer) this.map.removeLayer(this.currentlayer);
      this.map.addLayer(this.currentlayer = layer);
      this.currentlayername = name;
    }
  }

  OfflineProvider.prototype = new Provider;
  OfflineProvider.prototype.constructor = GoogleProvider;
  function OfflineProvider() { Provider.apply(this, arguments); this.Setup(); this.name = "Offline" }
  OfflineProvider.prototype.Setup = function () {
    var defaultStartMap = "Offline Map";
    this.baseLayers = {
      'Offline Map': _offlinemap
    };

    this.SetLayer("Offline Map");
  }
  window.OfflineProvider = OfflineProvider;


  GoogleProvider.prototype = new Provider;
  GoogleProvider.prototype.constructor = GoogleProvider;
  function GoogleProvider() { Provider.apply(this, arguments); this.Setup(); this.name = "Google"}
  GoogleProvider.prototype.Setup = function () {
    var defaultStartMap = "StreetMap";

    ///////////////////////
    // Google Map Layers //
    ///////////////////////
    this.baseLayers = {
      "StreetMap": new L.Google('ROADMAP'),
      "Satellite": new L.Google('SATELLITE'),
      "Hybrid": new L.Google('HYBRID'),
      "Terrain": new L.Google('TERRAIN')
    };

    this.SetLayer(defaultStartMap);
  }
  window.GoogleProvider = GoogleProvider;

  OpenStreetMapProvider.prototype = new Provider;
  OpenStreetMapProvider.prototype.constructor = OpenStreetMapProvider;
  function OpenStreetMapProvider() { Provider.apply(this, arguments); this.Setup(); this.name = "OpenStreetMap" }
  OpenStreetMapProvider.prototype.Setup = function () {

    this.baseLayers = {
      'Default': L.tileLayer.provider('OpenStreetMap.Mapnik').addTo(map),
      'German Style': L.tileLayer.provider('OpenStreetMap.DE'),
      'Black and White': L.tileLayer.provider('OpenStreetMap.BlackAndWhite'),
      'H.O.T.': L.tileLayer.provider('OpenStreetMap.HOT')
    }

    //this.control = L.control.layers(this.baseLayers);
    //this.control.addTo(this.map);
  }
  window.OpenStreetMapProvider = OpenStreetMapProvider;

  EsriProvider.prototype = new Provider;
  EsriProvider.prototype.constructor = EsriProvider;
  function EsriProvider() { Provider.apply(this, arguments); this.Setup(); this.name = "Esri" }
  EsriProvider.prototype.Setup = function () {

    this.baseLayers = {
      'StreetMap': L.tileLayer.provider('Esri.WorldStreetMap').addTo(this.map),
      'Satellite': L.tileLayer.provider('Esri.WorldImagery'),

      'DeLorme': L.tileLayer.provider('Esri.DeLorme'),
      'WorldTopoMap': L.tileLayer.provider('Esri.WorldTopoMap'),
      'WorldTerrain': L.tileLayer.provider('Esri.WorldTerrain'),
      'WorldShadedRelief': L.tileLayer.provider('Esri.WorldShadedRelief'),
      'WorldPhysical': L.tileLayer.provider('Esri.WorldPhysical'),
      'OceanBasemap': L.tileLayer.provider('Esri.OceanBasemap'),
      'NatGeoWorldMap': L.tileLayer.provider('Esri.NatGeoWorldMap'),
      'WorldGrayCanvas': L.tileLayer.provider('Esri.WorldGrayCanvas')
    }

    //this.control = L.control.layers(this.baseLayers);
    //this.control.addTo(this.map);
  }
  window.EsriProvider = EsriProvider;
}